﻿#region File Description
//-----------------------------------------------------------------------------
// AnimatedTexture.cs
//
// Microsoft XNA Community Game Platform
// Copyright (C) Microsoft Corporation. All rights reserved.
//
// modified by Kyoung Shin Park @ Dankook University (2011/4/25)
//-----------------------------------------------------------------------------
#endregion

using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace Sprite
{
    public class AnimatedTexture
    {
        private int framecount;     // number of animation frames
        private Texture2D texture;
        private float TimePerFrame;
        private int Frame;          // current frame #
        private float TotalElapsed;
        private bool Paused;

        private int framecountX;    // number of animation frames in X & Y
        private int framecountY;
        private int currFrameX;     // current Frame's X & Y index
        private int currFrameY;
        private int FrameWidth;     // only one frame's width & height
        private int FrameHeight;

        public float Rotation, Scale, Depth;
        public Vector2 Origin;

        public AnimatedTexture(Vector2 origin, float rotation, float scale, float depth)
        {
            this.Origin = origin;
            this.Rotation = rotation;
            this.Scale = scale;
            this.Depth = depth;
        }

        // load the n x 1 tiled image
        public void Load(ContentManager content, string asset, int frameCount, int framesPerSec)
        {
            framecountX = frameCount;
            framecountY = 1;

            framecount = frameCount;
            texture = content.Load<Texture2D>(asset);
            TimePerFrame = (float)1 / framesPerSec;
            Frame = 0;
            TotalElapsed = 0;
            Paused = false;

            FrameWidth = texture.Width / framecount;
            FrameHeight = texture.Height;
        }

        // load the m x n tiled image
        public void Load(ContentManager content, string asset, int frameCountX, int frameCountY, int framesPerSec)
        {
            framecountX = frameCountX;
            framecountY = frameCountY;

            framecount = frameCountX * frameCountY;
            texture = content.Load<Texture2D>(asset);
            TimePerFrame = (float)1 / framesPerSec;
            Frame = 0;
            TotalElapsed = 0;
            Paused = false;

            FrameWidth = texture.Width / frameCountX;
            FrameHeight = texture.Height / frameCountY;
        }

        // update frame #
        public void UpdateFrame(float elapsed)
        {
            if (Paused)
                return;
            TotalElapsed += elapsed;
            if (TotalElapsed > TimePerFrame)
            {
                Frame++;
                // Keep the Frame between 0 and the total frames, minus one.
                Frame = Frame % framecount;
                TotalElapsed -= TimePerFrame;
            }
        }

        // draw the frame #th image
        public void DrawFrame(SpriteBatch batch, Vector2 screenPos)
        {
            DrawFrame(batch, Frame, screenPos);
        }

        public void DrawFrame(SpriteBatch batch, int frame, Vector2 screenPos)
        {
            currFrameX = frame % framecountX;
            currFrameY = frame / framecountX;
            Rectangle sourcerect = new Rectangle(FrameWidth * currFrameX, FrameHeight * currFrameY, FrameWidth, FrameHeight);
            batch.Draw(texture, screenPos, sourcerect, Color.White, Rotation, Origin, Scale, SpriteEffects.None, Depth);
        }

        public bool IsPaused
        {
            get { return Paused; }
        }

        public void Reset()
        {
            Frame = 0;
            TotalElapsed = 0f;
        }

        public void Stop()
        {
            Pause();
            Reset();
        }

        public void Play()
        {
            Paused = false;
        }

        public void Pause()
        {
            Paused = true;
        }
    }
}
