using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
using Sprite;

namespace XNASpriteAnimation
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        // sprite texture
        Texture2D background;
        AnimatedTexture texture1;
        AnimatedTexture texture2;
        AnimatedTexture texture3;

        // input
        GamePadState currentGamePadState;
        GamePadState prevGamePadState;
        KeyboardState currentKeyboardState;
        KeyboardState prevKeyboardState;
        Vector2 position1 = new Vector2(0, 0);
        Vector2 scaling = new Vector2(0.8f, 0.8f);
        Vector2 position2 = new Vector2(200, 200);
        Vector2 position3 = new Vector2(300, 300);

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
            texture1 = new AnimatedTexture(Vector2.Zero, 0.0f, 1.0f, 0.5f);
            texture2 = new AnimatedTexture(Vector2.Zero, 30.0f, 3.0f, 0.5f);
            texture3 = new AnimatedTexture(Vector2.Zero, 0.0f, 1.5f, 0.5f);
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // background sprite texture
            background = Content.Load<Texture2D>("South_Park_Wallpaper");

            // animated sprite texture
            texture1.Load(Content, "goblintiles", 3, 3, 2); // 3x3 gobline
            position1.X = GraphicsDevice.Viewport.Width / 2;
            position1.Y = GraphicsDevice.Viewport.Height / 2;
            texture2.Load(Content, "shipanimated", 4, 2);   // 4x1 ship
            texture3.Load(Content, "cavemen", 4, 4, 3); // 4x4 cavemen
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            // TODO: Add your update logic here
            HandleInput();

            float elapsed = (float)gameTime.ElapsedGameTime.TotalSeconds;
            texture1.UpdateFrame(elapsed);
            texture2.UpdateFrame(elapsed);
            texture3.UpdateFrame(elapsed);

            base.Update(gameTime);
        }

        /// <summary>
        /// Determines which primitive to draw based on input from the keyboard
        /// or game pad.
        /// </summary>
        protected void HandleInput()
        {
            CheckGamePadInput();
            CheckKeyboardInput();
        }

        private void CheckGamePadInput()
        {
            prevGamePadState = currentGamePadState;
            currentGamePadState = GamePad.GetState(PlayerIndex.One);
        }

        private void CheckKeyboardInput()
        {
            prevKeyboardState = currentKeyboardState;
            currentKeyboardState = Keyboard.GetState(PlayerIndex.One);

            if (currentKeyboardState.IsKeyDown(Keys.Escape))
            {
                this.Exit();
            }
            else if (currentKeyboardState.IsKeyDown(Keys.Right))
            {
                position1.X++;
            }
            else if (currentKeyboardState.IsKeyDown(Keys.Left))
            {
                position1.X--;
            }
            else if (currentKeyboardState.IsKeyDown(Keys.Up))
            {
                position1.Y--;   // x+ to the right, y+ down
            }
            else if (currentKeyboardState.IsKeyDown(Keys.Down))
            {
                position1.Y++;   // x+ to the right, y+ down
            }
            else if (currentKeyboardState.IsKeyDown(Keys.Space)
                && !prevKeyboardState.IsKeyDown(Keys.Space))
            {
                if (texture1.IsPaused)
                    texture1.Play();
                else
                    texture1.Pause();
            }
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            // TODO: Add your drawing code here
            spriteBatch.Begin();
            spriteBatch.Draw(background, Vector2.Zero, null, Color.White, 0.0f, Vector2.Zero, scaling, SpriteEffects.None, 0);
            texture1.DrawFrame(spriteBatch, position1);
            texture2.DrawFrame(spriteBatch, position2);
            texture3.DrawFrame(spriteBatch, position3); 
            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
